import os,os.path,string,sys
import cherrypy
import coral
#from pluginCondDBPyInterface import *
from CondCore.TagCollection import Node,tagInventory,TagTree,DBCopy
from Common import config
import pickle

class GlobalTag(object):
    def __init__(self,configfilepath):
        self.__config=config.Config()
        self.__config.from_ini(configfilepath)
        self.__authpath=self.__config.common.auth_path
        self.__datasavedir=self.__config.common.datasave_dir
        self.__globaltagconfig=self.__config.globaltag.config_file
        self.__servicemap=self.__getServices()
        if len(self.__servicemap)==0:
            raise ValueError('no service is configured')
        
        #self.__coralcontext=coral.Context()
        #self.__coralcontext.setVerbosityLevel( 'ERROR' )
        self.__svc=coral.ConnectionService()
        self.__svc.configuration().setDefaultAuthenticationService('CORAL/Services/XMLAuthenticationService')
        os.environ['CORAL_AUTH_PATH']=self.__authpath
        self.__overview={}

    @cherrypy.expose
    def index(self):
        """Globaltag Top level page
        """
        return "GLOBALTAGPAGE!!!"
    
    @cherrypy.expose
    def overview(self):    
        """Overview the inventory and trees in all db instances
           The result is a dictionary: {connectstring:(basetaglist,treenamelist)}
        """
        self.__overview={}
        for dbconnect in self.__servicemap:
            dbconnect='oracle://'+dbconnect
            try:
                session=self.__svc.connect(dbconnect,accessMode=coral.access_Update)
            except :
                print 'unable to connect '+dbconnect+' skip \n'
                continue
            try:
                alltagsflat=[]
                trees=[]
                inv=tagInventory.tagInventory(session)
                alltags=inv.getAllEntries()                
                #print alltags
                session.transaction().start(True)
                tbs=session.nominalSchema().listTables()
                session.transaction().commit()
                for t in alltags:
                   tginfo={}
                   tginfo['tagid']=t.tagid
                   tginfo['tagname']=t.tagname
                   tginfo['objectname']=t.objectname
                   tginfo['pfn']=t.pfn
                   tginfo['recordname']=t.recordname
                   tginfo['labelname']=t.labelname
                   alltagsflat.append(tginfo)
                #print 'alltagsflat',alltagsflat
                for tbname in tbs:
                   if tbname.find('TAGTREE_TABLE_')!=-1:
                      treename=self.__gettreename(tbname)
                      trees.append(treename)
                #print trees
            except:
                #if schema is empty or any reading problems, return empty result                
                self.__overview[dbconnect]=([],[])
                
            self.__overview[dbconnect]=(alltagsflat,trees)
        #print 'services',self.__overview.keys()
        return pickle.dumps(self.__overview)
    
    def __getServices(self):
        """Get global tag db instance map from config file
        """
        sectionName='GLOBALTAG'
        #print 'globaltag.py config file in use ',self.__globaltagconfig
        try:
            globaltagconfig=config.Config()
            globaltagconfig.from_ini(self.__globaltagconfig)
            globaltagproperty=getattr(globaltagconfig,sectionName)
        except AttributeError:
            print 'No configuration found for',sectionName
            return []
        try:
            servicestr=getattr(globaltagproperty,'services')
            servicestr_clean=''.join(servicestr.strip(' ').split(' '))
            return servicestr_clean.split(',')
        except AttributeError:
            print 'services is not configured for',sectionName
            return []
        
    def __serviceToServlet(self,servicename):
        sectionName='GLOBALTAG'
        try:
            globaltagproperty=getattr(self.__config,sectionName)
        except AttributeError:
            print 'No configuration found for',sectionName
            return ''
        try:
            servletname=getattr(globaltagproperty,servicename)
            return servletname.strip(' ')
        except AttributeError:
            print 'servletname is not configured for',servicename
            return ''
        
    def __gettreename(self,tbname):
        """Given tree table name returns tree name
        """
        treename=tbname.split('_',2)[2]
        return treename
            
    @cherrypy.expose
    def getTree(self,treename,service,schema,viacache):
        """Get all nodes of a given tree. deliver result as pickled list
           It's leaf if lft=rgt-1. To be solved at client side
        """
        dbconnect=''
        if service=='sqlite': #for test purpose
            dbconnect='sqlite_file:'+schema+'.db'
        else:
            dbconnect='oracle://'+service+'/'+schema
        session=self.__svc.connect(dbconnect,accessMode=coral.access_Update)
        mytree=TagTree.tagTree(session,treename)
        inv=tagInventory.tagInventory(session)
        result=mytree.getSubtree('ROOT')
        flatresult=[]
        for node in result:
            flatnode={}
            flatnode["nodelabel"]=node.nodelabel
            if node.tagid != 0 :
               flatnode["tagid"]=node.tagid
               t=inv.getEntryById(node.tagid)
               flatnode["pfn"]=t.pfn
            flatnode["parentlabel"]=mytree.getNodeById(node.parentid).nodelabel
            flatresult.append(flatnode)
        #print flatresult
        del session
        return pickle.dumps(flatresult) 

    #####Editing operations#####

    @cherrypy.expose
    def cloneTree(self,sourcetreename,targettreename,service,schema,pfnreplace):
        """clone source tree to target tree with the possibility to replace the service part of the base pfn. If pfnreplace empty or None, source and target trees are identical.Example pfnreplace=frontier://abcd/efg . Note schema(account name is not included in the parameter. It is always kept.
        Output: pickled structure of [(oldtagid:newtagid)] if pfns are replaced; or [] if source tree is copied with no modification. 
        """
        dbconnect=''
        if service=='sqlite': #for test purpose
            dbconnect='sqlite_file:'+schema+'.db'
        else:
            dbconnect='oracle://'+service+'/'+schema
        session=self.__svc.connect(dbconnect,accessMode=coral.access_Update)
        newtree=TagTree.tagTree(session,targettreename)
        newtree.importFromTree(sourcetreename)
        if pfnreplace is None or len(pfnreplace)==0:
            del session
            return pickle.dumps([])
        
        newtaglinks=[]
        #print 'pfnreplace ',pfnreplace
        try:        
            inv=tagInventory.tagInventory(session)
            newtaglinks=inv.addEntriesReplaceService(pfnreplace)
            leafreplacement={}
            allLeafs=newtree.getAllLeaves()
            for leaf in allLeafs:
                oldtagid=leaf.tagid
                newtagid=0
                for item in newtaglinks:
                    if item[0]==oldtagid:
                        newtagid=item[1]
                if newtagid == 0:
                    raise "no replacement tagid found for tagid ",oldtagid
                leafreplacement[oldtagid]=newtagid
            newtree.replaceLeafLinks(leafreplacement)
            del session
        except Exception, e:
            print str(e)
            del session
        return pickle.dumps(newtaglinks)
    
    @cherrypy.expose
    def deleteNode(self,treename,nodename,service,schema):
        """delete a given node. Other nodes under the hierarchy are rearranged
        """
        dbconnect='oracle://'+service+'/'+schema
        session=self.__svc.connect(dbconnect,accessMode=coral.access_Update)
        mytree=TagTree.tagTree(session,treename)
        mytree.deleteNode(nodename)
        return

    @cherrypy.expose
    def deleteBranch(self,treename,nodename,service,schema):
        """delete a branch under the given node. 
        """
        dbconnect='oracle://'+service+'/'+schema
        session=self.__svc.connect(dbconnect,accessMode=coral.access_Update)
        mytree=TagTree.tagTree(session,treename)
        mytree.deleteSubtree(nodename)
        return
        
    @cherrypy.expose
    def renameNodes(self,treename,service,schema,nodenamemap):
        """rename nodes.
        Input: pickled structure of {'oldnodelabel':oldnodelabelname,'newnodename':newnodelabelname}
        
        """
        #mynodemap=pickle.loads(nodenamemap)
        #if len(mynodemap)==0:
        #    return
        #s=repr(nodenamemap)
        n="{"+repr(nodenamemap)+"}"
        mynodemap=eval(n)
        #print 'mynodemap',mynodemap
        dbconnect=''
        if service=='sqlite': #for test purpose
            dbconnect='sqlite_file:'+schema+'.db'
        else:
            dbconnect='oracle://'+service+'/'+schema
        
        session=self.__svc.connect(dbconnect,accessMode=coral.access_Update)
        t=TagTree.tagTree(session,treename)
        t.renameNodes(mynodemap)
        del session
        return 

    @cherrypy.expose
    def insertNode(self,treename,service,schema,parentname,nodelabel,tagid):
        """Insert a new node to specified parent. \n
        Input: tree name
        Input: pickled new node dict {globalsince:unsigned long, globaltill:unsigned long, nodelabel:string}. \n
        Input: name of the parent node. New node can be inserted only if
        parentnode tagid==0 (not leaf). Please check parentnode.tagid!=0 from client side before calling this method. \n
        Output: result nodeid  
        """
        #print 'nodelabel: ',nodelabel
        #print 'tagid: ',tagid
        #print 'parentname: ',parentname
        #mynewnodedict=pickle.loads(nodedict)
        #mynewnodedict=pickle.loads(pickle.dumps(nodedict)
        mynewnode=Node.Node()
        if(len(tagid)!=0 and int(tagid)!=0):
          mynewnode.tagid = int(tagid)
        else:
          mynewnode.tagid=0
        mynewnode.nodelabel=nodelabel
        dbconnect=''
        if service=='sqlite': #for test purpose
             dbconnect='sqlite_file:'+schema+'.db'
        else:
             dbconnect='oracle://'+service+'/'+schema
        
        session=self.__svc.connect(dbconnect,accessMode=coral.access_Update)
        t=TagTree.tagTree(session,treename)
        nodeid=t.insertNode(mynewnode,parentname)
        del session
        return nodeid

    @cherrypy.expose
    def exportAll(self,service,schema,sqlitefilename):
        """export selected tag db to sqlite file
        Input: sqlitefilename
        """
        datafile=os.path.join(self.__datasavedir,sqlitefilename)
        destconnect='sqlite_file://'+datafile
        sourceconnect=''
        if service=='sqlite': #for test purpose
             sourceconnect='sqlite_file:'+schema+'.db'
        else:
             sourceconnect='oracle://'+service+'/'+schema
        sourcesession=self.__svc.connect(sourceconnect,accessMode=coral.access_Update )
        destsession=self.__svc.connect(destconnect,accessMode=coral.access_Update)
        try:
            dbcp=DBCopy.DBCopy(sourcesession,destsession,1024)
            del sourcesession
            del destsession
        except Exception, e:
            print str(e)
            del sourcesession
            del destsession
        return static.serve_file(datafile,"application/x-download", "attachment",os.path.basename(datafile))

    @cherrypy.expose
    def exportTrees(self,treenames,service,schema,sqlitefilename):
        """export selected trees to sqlite file
        """
        dbconnect='oracle://'+service+'/'+schema
        datafile=os.path.join(self.__datasavedir,sqlitefilename)
        destconnect='sqlite_file://'+datafile
        sourcesession=self.__svc.connect(dbconnect,accessMode=coral.access_Update )
        destsession=self.__svc.connect(destconnect,accessMode=coral.access_Update )
        try:
            dbcp=DBCopy.DBCopy(sourcesession,destsession,1024)
            dbcp.copyTrees(treenames)
            del sourcesession
            del destsession
        except Exception, e:
            print str(e)
            del sourcesession
            del destsession
        return static.serve_file(datafile,"application/x-download", "attachment",os.path.basename(datafile))
