import os,os.path,string,commands
import cherrypy
#import re
from Common import config,DetConfig
import pluginCondDBPyInterface as CondDB
from cherrypy.lib import static
import datetime
class IOV(object):
    def __init__(self,configfilepath):
        self.__config=config.Config()
        self.__config.from_ini(configfilepath)
        self.__authpath=self.__config.common.auth_path
        self.__datasavedir=self.__config.common.datasave_dir
        #print 'detector_config ',self.__config.common.detector_conf
        self.__detconfig=DetConfig.DetConfig(self.__config.common.detector_conf)
        self.__logdb=str(self.__config.iov.log_conn)

    @cherrypy.expose
    def index(self):
        """Top level page
        """
        #print self.__authpath
        return "IOVPAGE!!!"
        
    def __parseListMappingResult(self,inputdata):
        """Parse the result from pool_list_object_relational_mapping
        """
        myr={}
        for i in inputdata:
           r=i.strip(' ').split('\n')[0]
           result=r.strip(' ').split('\n')
           for j in result:
             fields=j.split(' ')
             mappingversion=fields[0].strip(' ')
             container=fields[-1].strip(' ')
             if len(container)==0:
                 continue
             #print 'container',container
             #print 'mappingversion ',mappingversion
             myr[container]=mappingversion
        return myr
    
    def __getconnectstr(self,det,service,schema,viacache):
        """Assemble full connect string with given parameters
        """
        if len(viacache)==0:
            connect_str=''
            if (service.find(":"))==-1 :
               connect_str='oracle://'
            connect_str+=service+'/'+schema          
        else:            
            knownservices=self.__detconfig.services(det)
            connect_str=''
            if '/'.join([service,schema]) not in knownservices:
               return ''
            servletname=self.__detconfig.serviceToServlet(det,service)
            connect_str='frontier://cmsfrontier.cern.ch:8000/'+servletname+'/'+schema
        return connect_str
    
    @cherrypy.expose
    def getTags(self,det,service,schema,viacache):
        """Get list of tags
        """
        #p = re.compile('\w*[\a\n\n\v\t\r();:\'"|~`]\w*')
        #if (p.match(det)!=None or p.match(service) != None) or (p.match(schema) != None) :
        #    return "internal error - unauthourized access"
        connect_str=self.__getconnectstr(det,service,schema,viacache)
        print "connect to ", connect_str
        if len(connect_str)==0:
            return "Application Server Error: requested service "+service+" "+schema+"is not configured"
        print connect_str
       	rdbms = CondDB.RDBMS(str(self.__authpath))
        rdbms.setLogger(self.__logdb)
        db = rdbms.getDB(connect_str)              
        tags = db.allTags()
        result=''
        for entry in tags.split():
            iov = db.iov(entry)
            log = db.lastLogEntry(entry).getState()
            print log
            result += entry.strip() + " " + str(iov.size()) + ';'
        iov=0
        db=0
        return result
        
    @cherrypy.expose
    def getIOVForTag(self,det,service,schema,tag,viacache):
        """Get iov info for given tag
        """
        #p = re.compile('\w*[\a\n\n\v\t\r();:\'"|~`]\w*')
        #if (p.match(service) != None) or (p.match(schema) != None) or (p.match(tag)!= None):
        #    return "internal error - unauthourized access"
        connect_str=self.__getconnectstr(det,service,schema,viacache)
        if len(connect_str)==0:
            return "Application Server Error: requested service "+service+" "+schema+"is not configured"
        cmd=['cmscond_list_iov','-t',tag,'-c',connect_str]
        if connect_str.find('oracle://') != -1:
            cmd.append('-P')
            cmd.append(self.__authpath)
        cmdstr=' '.join(cmd)
        cmdout = commands.getoutput(cmdstr)
        results = string.split(cmdout,'\n')
        result=''
        for entry in results:
            result += entry.strip() +';'
        return result

    @cherrypy.expose
    def exportIOV(self,det,service,schema,containername,inputTag,destTag,firstSince,lastTill,viacache):
        """export iov in CMSSW20x format
        """
        h=(inputTag+firstSince+lastTill+viacache).__hash__()
        filename='_'.join([service,schema,containername,destTag])+str(h)
        today=datetime.date.today()
        datadir=os.path.join(self.__datasavedir,str(today))
        if os.path.exists(datadir) is False:
            os.mkdir(datadir)
        datafile=os.path.join(datadir,filename+'.db')
        mappingfile=os.path.join(datadir,filename+'.xml')
        if os.path.exists(datafile) is True :
            return static.serve_file(datafile, "application/x-download", "attachment",os.path.basename(datafile))

        destconnect='sqlite_file://'+datafile
        #print 'destconnect',destconnect
        connect_str=self.__getconnectstr(det,service,schema,viacache)
        if len(connect_str)==0:
            return "Application Server Error: requested service "+service+" "+schema+"is not configured"
        sourceconnect=connect_str
        #print 'det',det
        dict=self.__detconfig.dict_name(det)
        #print 'dict',dict

        cmd1=['pool_list_object_relational_mapping','-l','-c',sourceconnect]
        #parse result get mapping version for given container
        #mappingfile='mapping.xml'
        cmd2=['pool_retrieve_object_relational_mapping','-c',sourceconnect,'-o',mappingfile]
        #build mapping in the dest db
        cmd3=['pool_build_object_relational_mapping','-f',mappingfile,'-c',destconnect,'-d',dict]
        cmd4 = ['cmscond_export_iov','-s',sourceconnect,'-d',destconnect,'-D',dict,'-t',destTag]
        if connect_str.find('oracle://') != -1:
            os.environ['CORAL_AUTH_PATH']=self.__authpath
            #cmd1.append('-u')
            #cmd1.append(user)
            #cmd1.append('-p')
            #cmd1.append(passwd)
            #cmd2.append('-u')
            #cmd2.append(user)
            #cmd2.append('-p')
            #cmd2.append(passwd)
            cmd4.append('-P')
            cmd4.append(self.__authpath)
        if len(inputTag)!=0:
            cmd4.append('-i')
            cmd4.append(inputTag)
        if len(firstSince)!=0:
            cmd4.append('-b')
            cmd4.append(firstSince)
        if len(lastTill)!=0:
            cmd4.append('-e')
            cmd4.append(lastTill)
        
        cmdstr = ' '.join(cmd1)
        cmdout = commands.getoutput(cmdstr)
        #if cmdout.find('ERROR')!=-1:
        #    return "Application Server : ",cmdout
        ormappingmap=self.__parseListMappingResult( cmdout.split('\n') )
        #print 'ormappingmap',ormappingmap
        mappingversion=ormappingmap[containername]
        
        cmd2+=['-v',mappingversion]
        cmdstr = ' '.join(cmd2)
        cmdout = commands.getoutput(cmdstr)
        if cmdout.find('ERROR')!=-1:
            return "Application Server : ",cmdout
        #print cmdout
        cmdstr = ' '.join(cmd3)
        cmdout = commands.getoutput(cmdstr)
        if cmdout.find('ERROR') != -1:
            return 'Application server : ', cmdout
        cmdstr = ' '.join(cmd4)
        cmdout = commands.getoutput(cmdstr)
        if cmdout.find('ERROR')!=-1:
            return "Application Server : ",cmdout
        #print os.path.basename(datafile)
        #print 'datafile ',datafile
        #print 'data path ',os.path.basename(datafile)
        return static.serve_file(datafile, "application/x-download", "attachment",os.path.basename(datafile))

    
    @cherrypy.expose
    def getFile(self,det,service,schema,tag,viacache,payload,filename):
        """Export selected tag in CMSSW18x compatible format
        """
        datafile=os.path.join(self.__datasavedir,filename)
        destconnect='sqlite_file://'+datafile

        connect_str=self.__getconnecstr(det,service,schema,viacache)
        if len(connect_str)==0:
            return "Application Server Error: requested service "+service+" "+schema+"is not configured"
        sourceconnect=connect_str
        #print 'det',det
        dict=self.__detconfig.dict_name(det)
        cmd = ['cmscond_export_iov','-s',sourceconnect,'-d',destconnect,'-D',dict,'-n',payload,'-p',self.__authpath,'-t',tag]
        cmdstr=' '.join(cmd)
        #print cmd
        cmdout = commands.getoutput(cmdstr)
        #print cmdout
        #print os.path.basename(datafile)
        return static.serve_file(datafile, "application/x-download", "attachment",os.path.basename(datafile))

    
    @cherrypy.expose
    def getPlot(self):
        """a test to understand few thing
        """
	filename = "bha.jpg"
        datafile=os.path.join(self.__datasavedir,filename)
    	print datafile
        print os.path.basename(datafile)
        return static.serve_file(datafile, "application/x-download", "attachment",os.path.basename(datafile))

