import os,os.path,string,commands
import cherrypy
#import re
from Common import config,DetConfig
import pluginCondDBPyInterface as CondDB
from CondCore.Utilities import iovInspector as inspect

from cherrypy.lib import static
import datetime
import pickle
class IOV(object):
    def __init__(self,configfilepath):
        self.__config=config.Config()
        self.__config.from_ini(configfilepath)
        self.__authpath=self.__config.common.auth_path
        self.__datasavedir=self.__config.common.datasave_dir
        self.__detconfig=DetConfig.DetConfig(self.__config.common.detector_conf)
        self.__logdb=str(self.__config.iov.log_conn)
        self.__cmsframe=CondDB.FWIncantation
        
    @cherrypy.expose
    def index(self):
        """Top level page
        """
        #print self.__authpath
        return "IOV inspection service"
    
    
    def __getconnectstr(self,det,service,schema,viacache):
        """Assemble full connect string with given parameters
        """
        connect_str=''
        if len(viacache)==0:
            if (service.find(":"))==-1 :
               connect_str+='oracle://'
            connect_str+=service+'/'+schema          
        else:            
            #knownservices=self.__detconfig.services(det)
            #if '/'.join([service,schema]) not in knownservices:
            #   return ''
            servletname=self.__detconfig.serviceToServlet(det,service)
            connect_str+='frontier://cmsfrontier.cern.ch:8000/'+servletname+'/'+schema
        if len(connect_str)==0:
            return "Application Server Error: requested service "+service+" "+schema+"is not configured"
        return connect_str
    
    @cherrypy.expose
    def getTags(self,det,service,schema,viacache):
        """Get list of tags
        """
        connect_str=self.__getconnectstr(det,service,schema,viacache)
       	rdbms = CondDB.RDBMS(str(self.__authpath))
        #rdbms.setLogger(self.__logdb)
        db = rdbms.getDB(str(connect_str))              
        tags = db.allTags()
        result=''
        for entry in tags.split():
            #iov = db.iov(entry)
            #log = db.lastLogEntry(entry).getState()
            #print log
            #result += entry.strip() + " " + str(iov.size()) + ';'
            result += entry.strip()+';'
        iov=0
        db=0
        rdbms = 0
        return result
    
    @cherrypy.expose
    def getPayloadSummaryForTag(self,det,service,schematag,since,till,head,tail, viacache):
        """Get payload summary info for given tag in the given iov range
        """
        if (0==len(till)) : till = 0
        if (0==len(since)) : since = 0
        if (0==len(head)) : head = 0
        if (0==len(tail)) : tail = 0
        connect_str=self.__getconnectstr(det,service,schema,viacache)
        rdbms = CondDB.RDBMS(str(self.__authpath))
        db = rdbms.getDB(str(connect_str))
        iov = inspect.Iov(db,tag,long(since),long(till),long(head),long(tail))
        ret = pickle.dumps(iov.summaries())
        iov=0
        db=0
        rdbms = 0
        return ret
        
    @cherrypy.expose
    def getIOVForTag(self,det,service,schema,tag, since, till, head, tail, viacache):
        """Get iov info for given tag in the given range
        """
        if (0==len(till)) : till = 0
        if (0==len(since)) : since = 0
        if (0==len(head)) : head = 0
        if (0==len(tail)) : tail = 0
    
        connect_str=self.__getconnectstr(det,service,schema,viacache)
        rdbms = CondDB.RDBMS(str(self.__authpath))
        #rdbms.setLogger(self.__logdb)
        db = rdbms.getDB(str(connect_str))
        # this one still crashes if tag is not present...
        iov = inspect.Iov(db,tag,long(since),long(till),long(head),long(tail))
        #iov = inspect.Iov(db,tag)
        timetype=iov.timetype()
        result=(str(timetype),iov.list())
        ret= pickle.dumps(result)
        iov=0
        db=0
        rdbms = 0
        return ret

    @cherrypy.expose
    def getWhatForTag(self,det,service,schema,tag,viacache):
        """get What() information defined by the payload
        """
        rdbms = CondDB.RDBMS(str(self.__authpath))
        connect_str=self.__getconnectstr(det,service,schema,viacache)
        db = rdbms.getDB(str(connect_str))
        what = pickle.dumps(inspect.extractorWhat(db,tag))
        db=0
        rdbms = 0
        return what

    @cherrypy.expose
    def getTrendForTag(self,det,service,schema,tag, since, till, extractor, viacache):
        """get trend data with the given trend extractor for a given tag in the given range.
        Output: pickled struct. [(since,till,value),()]
        """
        what = pickle.loads(extractor)
        if (0==len(till)) :
            since = 0
            till = 0
        connect_str=self.__getconnectstr(det,service,schema,viacache)
        rdbms = CondDB.RDBMS(str(self.__authpath))
        #rdbms.setLogger(self.__logdb)
        db = rdbms.getDB(std(connect_str))
        # this one still crashes if tag is not present...
        iov = inspect.Iov(db,tag,long(since),long(till))
        ret = pickle.dumps(iov.trend(what))
        iov=0
        db=0
        rdbms = 0
        return ret
    
    @cherrypy.expose
    def getPlot(self):
        """a test to understand few thing
        """
	filename = "bha.jpg"
        datafile=os.path.join(self.__datasavedir,filename)
    	#print datafile
        #print os.path.basename(datafile)
        return static.serve_file(datafile, "application/x-download", "attachment",os.path.basename(datafile))

